/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import java.util.List;

import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lms.vew.RDSException;
import de.bsvrz.kex.lms.vew.daten.RDSStatus.RDS_StatusEN;

/**
 * Datensatz einer RDS-Meldung.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class RDSMeldungsDaten extends AbstractRDSInfoElement {
	/**
	 * XML-Tag RDS-Meldung.
	 */
	public static final String XML_ELEMENT_MELDUNG = "IFN";

	/**
	 * TIC Info XML Element: InfoGUID, Globaler eindeutiger Identifikator der
	 * Information.
	 */
	private static final String XML_ELEMENT_INFOGUID = "IID";
	/**
	 * Identifikator der Information.
	 */
	private final RDSText infoText = new RDSText(RDSMeldungsDaten.XML_ELEMENT_INFOGUID, "InfoGUID");

	/**
	 * Daten der Version.
	 */
	RDSVersion rdsVersion = new RDSVersion();

	/**
	 * Die ID der Meldung.
	 */
	private final RDSID rdsId = new RDSID();

	/**
	 * Erzeugt ein neues Objekt.
	 */
	public RDSMeldungsDaten() {
		super(RDSMeldungsDaten.XML_ELEMENT_MELDUNG, "");
		appendChild(infoText);
		appendChild(rdsId);
		appendChild(rdsVersion);
	}

	/**
	 * Erzeugt ein neues Objekt aus einem Knoten eines XML-Dokumentes.
	 *
	 * @param node
	 *            Knoten
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	public RDSMeldungsDaten(final Node node) throws RDSException {
		this();
		if ((node.getNodeType() != Node.ELEMENT_NODE)
				|| !node.getNodeName().equals(RDSMeldungsDaten.XML_ELEMENT_MELDUNG)) {
			throw new RDSException("falscher Knotentyp!");
		}
		einlesenXML(node);
	}

	@Override
	public void aktualisieren(final Data davdata) throws RDSException {
		for (final RDSInfoElement child : kinder) {
			child.aktualisieren(davdata);
		}
	}

	/**
	 * Erzeugt eine PID aus den Daten der Meldung.
	 *
	 * @return PID
	 */
	public String bildePid() {
		return "rds.lms." + infoText.getText();
	}

	/**
	 * Gibt die Identifikation der Meldung zur&uuml;ck.
	 *
	 * @return Identifikation der Meldung
	 */
	public String getIdentifikation() {
		return infoText.getText();
	}

	/**
	 * Gibt die IDNumber der Meldung zur&uuml;ck.
	 *
	 * @return IDNumber der Meldung
	 */
	public String getIDNumber() {
		return rdsId.getIDNumber();
	}

	/**
	 * Gibt die IDOrganisation der Meldung zur&uuml;ck.
	 *
	 * @return IDOrganisation der Meldung
	 */
	public String getIDOrganisation() {
		return rdsId.getIDOrganisation();
	}

	/**
	 * Gibt alle in der Meldung referenzierten Locationcodes zur&uuml;ck.
	 *
	 * @return Liste von Locationcodes
	 */
	public List<Integer> getLocationCodes() {
		return rdsVersion.getLocationCodes();
	}

	/**
	 * Gibt den Status der Meldungzur&uuml;ck.
	 *
	 * @return Status
	 */
	public RDS_StatusEN getMeldungsStatus() {
		return rdsVersion.verwaltungsInformationen.getMeldungsStatus();
	}

	/**
	 * Gibt die Versionsnummer der Meldung zur&uuml;ck.
	 *
	 * @return Versionsnummer
	 */
	public int getVersion() {
		return rdsVersion.verwaltungsInformationen.getVersion();
	}

	/**
	 * Test, ob die Meldung g&uuml;ltig ist.
	 *
	 * @return true wenn g&uuml;ltig, sonst false
	 */
	public boolean istZeitGueltig() {
		return rdsVersion.istZeitGueltig();
	}

	/**
	 * Test, ob die Meldung verarbeitet werden muss. Meldungen mit Status
	 * 'Aufhebung' oder 'L&ouml;schung' werden immer verarbeitet. F&uuml;r alle
	 * anderen Stati wird gepr&uuml;ft, ob die Ablaufzeit bereits erreicht ist.
	 *
	 * @return true, wenn die Version (noch) g&uuml;ltig ist, sonst false
	 */
	public boolean istZuVerarbeiten() {
		if ((getMeldungsStatus() == RDS_StatusEN.Aufhebung) || (getMeldungsStatus() == RDS_StatusEN.Loeschung)) {
			return true;
		}

		return istZeitGueltig();
	}

	/**
	 * Setzt die Ablaufzeit der Meldung.
	 *
	 * @param neueAblaufzeit
	 *            neue Ablaufzeit
	 */
	public void setAblaufzeit(final long neueAblaufzeit) {
		rdsVersion.verwaltungsInformationen.setAblaufzeit(neueAblaufzeit);
	}

	/**
	 * Setzt die IDNumber der Meldung.
	 *
	 * @param nummer
	 *            IDNumber
	 */
	public void setIDNumber(final String nummer) {
		rdsId.setIDNumber(nummer);
	}

	/**
	 * Setzt die IDOrganisation der Meldung.
	 *
	 * @param organisation
	 *            IDOrganisation der Meldung
	 */
	public void setIDOrganisation(final String organisation) {
		rdsId.setIDOrganisation(organisation);
	}

	/**
	 * Setzt den Status der Meldung.
	 *
	 * @param neuerstatus
	 *            neuer Status
	 */
	public void setMeldungsStatus(final RDS_StatusEN neuerstatus) {
		rdsVersion.verwaltungsInformationen.setMeldungsStatus(neuerstatus);
	}

	/**
	 * Setzt die Identifikation der Meldung.
	 *
	 * @param identifikation
	 *            Identifikation
	 */
	public void setzeIdentifikation(final String identifikation) {
		infoText.setText(identifikation);
	}
}
